package com.huimv.eartag2.process.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.EartagEnvEntity;
import com.huimv.eartag2.common.dao.entity.EartagHeartbeatEntity;
import com.huimv.eartag2.common.dao.entity.EartagRawEntity;
import com.huimv.eartag2.common.dao.repo.EartagEnvRepo;
import com.huimv.eartag2.common.dao.repo.EartagHeartbeatRepo;
import com.huimv.eartag2.common.dao.repo.EartagRawRepo;
import com.huimv.eartag2.process.service.IDeviceService;
import com.huimv.eartag2.process.utils.DateUtil;
import com.huimv.eartag2.process.utils.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceServiceImpl implements IDeviceService {
    @Autowired
    private MathUtil mathUtil;
    @Autowired
    private DateUtil dateUtil;
    @Autowired
    private EartagRawRepo rawRepo;
    @Autowired
    private EartagHeartbeatRepo heartbeatRepo;
    @Autowired
    private EartagEnvRepo envRepo;

    /**
     * @Method      : saveDeviceHeartbeat
     * @Description : 保存设备心跳数据
     * @Params      : [dataJo]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/2/15       
     * @Time        : 14:39
     */
    @Override
    public void saveDeviceHeartbeat(JSONObject dataJo) {
        String deviceCode = dataJo.getString("device");
        EartagHeartbeatEntity addHeartbeatEntity = new EartagHeartbeatEntity();
        addHeartbeatEntity.setDeviceCode(deviceCode);
        addHeartbeatEntity.setAddTime(new Timestamp(new Date().getTime()));
        heartbeatRepo.saveAndFlush(addHeartbeatEntity);
    }

    /**
     * @Method      : saveDeviceTemp
     * @Description : 保存设备温度数据
     * @Params      : [dataJo]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/2/15       
     * @Time        : 14:39
     */
    @Override
    public void saveDeviceTemp(JSONObject dataJo) {
        String device = dataJo.getString("device");
        String temp = dataJo.getString("temp");
        EartagEnvEntity envEntity = new EartagEnvEntity();
        envEntity.setDeviceCode(device);
        envEntity.setTemp1(Integer.parseInt(temp));
        envEntity.setTemp2(0F);
        envEntity.setAddTime(new Timestamp(new Date().getTime()));
        envRepo.saveAndFlush(envEntity);
    }

    /**
     * @Method      : saveRawdata
     * @Description : 保存原始数据
     * @Params      : [askText]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/2/9
     * @Time        : 13:38
     */
    @Override
    public void saveRawdata(String askText) throws ParseException {
//        log.info("#开始解析Ask请求数据>>" + askText.trim().replace("\n", ""));
        String[] dataArray = askText.split("\\+");
        String cmdHeader = dataArray[0];
        Map rawMap = new HashMap();
        if (cmdHeader.trim().equalsIgnoreCase("hm")) {
            //采集器原始数据
            String device = dataArray[1];
            String cmd = dataArray[2];
            rawMap.put("cmdHeader", cmdHeader);
            rawMap.put("device", device);
            rawMap.put("cmd", cmd);
            rawMap.put("ask", askText);
        } else if (cmdHeader.trim().equalsIgnoreCase("zj")) {
            //采集器原始数据
            String device = dataArray[1];
            rawMap.put("cmdHeader", cmdHeader);
            rawMap.put("device", device);
            rawMap.put("cmd", "");
            rawMap.put("ask", askText);
            rawMap.put("addTime", dateUtil.getTodayMissionText());
        } else {
            //采集器原始数据
            String device = dataArray[1];
            rawMap.put("cmdHeader", cmdHeader);
            rawMap.put("device", device);
            rawMap.put("cmd", "");
            rawMap.put("ask", askText);
        }
        //保存原始数据
        //类型、设备、命令、数据内容、日期
        EartagRawEntity addRawEntity = new EartagRawEntity();
        addRawEntity.setCmdHeader(rawMap.get("cmdHeader").toString());
        addRawEntity.setDevice(rawMap.get("device").toString());
        addRawEntity.setCmd(rawMap.get("cmd").toString());
        addRawEntity.setAsk(rawMap.get("ask").toString());
        addRawEntity.setAddTime(new Timestamp(new Date().getTime()));
        rawRepo.saveAndFlush(addRawEntity);
    }
}
