package com.huimv.eartag2.process.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.EartagDataEntity;
import com.huimv.eartag2.common.dao.repo.EartagDataRepo;
import com.huimv.eartag2.process.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class EartagServiceImpl implements IEartagService {
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private EartagDataRepo eartagDataRepo;

    /**
     * @Method : countAct
     * @Description : 计算act数值
     * @Params : [deviceCode, nowAct0]
     * @Return : java.lang.Integer
     * @Author : ZhuoNing
     * @Date : 2022/1/18
     * @Time : 15:25
     */
    @Override
    public Integer countAct(String deviceCode, String nowAct) {
        //读取hash
        Object actObj = redisTemplate.opsForHash().get(deviceCode, "act");
        if (actObj == null) {
            //--初始化最新的redis记录
            initRedisObj(deviceCode);
            Object lastAct = redisTemplate.opsForHash().get(deviceCode, "act");
            if (lastAct == null) {
                log.error("#--- redis数据库有问题，请检查redis是否能正常连接 ---# ");
                return 0;
            }
        }
        Object lastAct = redisTemplate.opsForHash().get(deviceCode, "act");
        Integer act1 = Integer.parseInt(nowAct) - Integer.parseInt(lastAct.toString());
//        redisTemplate.opsForHash().put(deviceCode, "act1", act1);
        return act1;
    }

    /**
     * @Author      : ZhuoNing
     * @Date        : 2022/2/9
     * @Time        : 14:33
     */
    @Override
    public void saveEartag(JSONObject eartagJo) {
        EartagDataEntity dataEntity = new EartagDataEntity();
        dataEntity.setCmdHeader(eartagJo.getString("cmdHeader"));
        dataEntity.setDevice(eartagJo.getString("device"));
        dataEntity.setEarmark(eartagJo.getString("earmark"));
        dataEntity.setBat(Integer.parseInt(eartagJo.getString("bat")));
        dataEntity.setEarTemp(Integer.parseInt(eartagJo.getString("earTemp")));
        dataEntity.setEarTemp1(eartagJo.getFloat("earTemp1"));
        dataEntity.setEnvTemp(Integer.parseInt(eartagJo.getString("envTemp")));
        dataEntity.setEnvTemp1(eartagJo.getFloat("envTemp1"));
        dataEntity.setAct(Integer.parseInt(eartagJo.getString("act")));
        dataEntity.setAct1(Integer.parseInt(eartagJo.getString("act1")));
        dataEntity.setSignal1(Integer.parseInt(eartagJo.getString("signal")));
        dataEntity.setAskTime(getAskTime(eartagJo.getString("askTime")));
        dataEntity.setOther(eartagJo.getString("other"));
        dataEntity.setAddTime(new Timestamp(new Date().getTime()));
        eartagDataRepo.saveAndFlush(dataEntity);

        //最新的耳标数据覆盖老数据
        Map<String, String> map = new HashMap<>();
        map.put("cmdHeader", dataEntity.getCmdHeader());
        map.put("device", dataEntity.getDevice());
        map.put("earmark", dataEntity.getEarmark());
        map.put("bat", dataEntity.getBat().toString());
        map.put("earTemp", dataEntity.getEarTemp().toString());
        map.put("earTemp1", dataEntity.getEarTemp1().toString());
        map.put("envTemp", dataEntity.getEnvTemp().toString());
        map.put("envTemp1", dataEntity.getEnvTemp1().toString());
        map.put("act", dataEntity.getAct().toString());
        map.put("act1", dataEntity.getAct1().toString());
        map.put("signal1", dataEntity.getSignal1().toString());
        map.put("askTime", dataEntity.getAskTime());
        map.put("other", dataEntity.getOther());
        map.put("addTime", dataEntity.getAddTime().toString());
        //为hash结构设置多个键值对(hmset)
        redisTemplate.opsForHash().putAll(dataEntity.getDevice(), map);
    }

    //
    public String getAskTime(String askTime) {
        String newAskTime = askTime.substring(0, 4) + "-" + askTime.substring(4, 6) + "-" + askTime.substring(6, 8) + " " + askTime.substring(8, 10) + ":" + askTime.substring(10, 12) + ":" + askTime.substring(12, 14);
        return newAskTime;
    }

    /**
     * 初始化redis最新记录
     */
    private void initRedisObj(String deviceCode) {
        EartagDataEntity lastEartagDataEntity = eartagDataRepo.getLastEartagData(deviceCode);
        if (lastEartagDataEntity != null) {
            System.out.println("## lastEartagDataEntity.toString>>" + lastEartagDataEntity.toString());
            //--从数据库读取最新记录并覆盖redis
            Map<String, String> map = new HashMap<>();
            map.put("cmdHeader", lastEartagDataEntity.getCmdHeader());
            map.put("device", lastEartagDataEntity.getDevice());
            map.put("earmark", lastEartagDataEntity.getEarmark());
            map.put("bat", lastEartagDataEntity.getBat().toString());
            map.put("earTemp", lastEartagDataEntity.getEarTemp().toString());
            map.put("earTemp1", lastEartagDataEntity.getEarTemp1().toString());
            map.put("envTemp", lastEartagDataEntity.getEnvTemp().toString());
            map.put("envTemp1", lastEartagDataEntity.getEnvTemp1().toString());
            map.put("act", lastEartagDataEntity.getAct().toString());
            map.put("act1", lastEartagDataEntity.getAct1().toString());
            map.put("signal1", lastEartagDataEntity.getSignal1().toString());
            map.put("askTime", lastEartagDataEntity.getAskTime());
            map.put("other", lastEartagDataEntity.getOther());
            map.put("addTime", lastEartagDataEntity.getAddTime().toString());
            //为hash结构设置多个键值对(hmset)
            redisTemplate.opsForHash().putAll(deviceCode, map);
        }
    }
}
