package com.huimv.eartag2.process2.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.mq.Const;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.process2.service.ICacheService;
import com.huimv.eartag2.process2.service.IDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class DeviceListener {
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private IDeviceService deviceService;
    @Autowired
    private ICacheService cacheService;

    /**
     * @Method : processDeviceHeartbeat
     * @Description : 处理设备心跳数据
     * @Params : [RawMap]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/2/15
     * @Time : 8:56
     */
//    @RabbitListener(queues = Const.QUEUE_DEVICE_HEARTBEAT)
    @RabbitHandler
    public void processDeviceHeartbeat(Map DeviceMap) throws ParseException {
        System.out.println("222 <<<<<<<<<<<<<<< 处理设备心跳数据>>" + DeviceMap.toString());
        //--更新设备注册数据(MySQL,Redis)-
        //--保存设备在线统计数据(MySQL,Redis)-
        //--保存设备状态数据(MySQL,Redis)-
        JSONObject deviceJo = JSON.parseObject(DeviceMap.get("heartbeat").toString());
        String deviceCode = deviceJo.getString("device");
        System.out.println("deviceCode>>" + deviceCode);
        //获取牧场id
        String farmId = deviceService.getFarmIdByDeviceCode(deviceCode);
        String todayDateText = new DateUtil().getTodayDateText();
        Date todayDate = new Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());

        //判断该设备编号是否存在缓存在线集合当中；
        if (cacheService.isExistDeviceOnlineSet(farmId, deviceCode, todayDateText)) {
            //Step1:更新设备注册表[设置活动状态为1，设置活动状态更新时间]
            //Step2:更新在线统计[心跳数+1，心跳第一次上传时间，心跳最后一次时间]，预先判断该数据是否存在，如果不存在要先添加记录，如果存在就修改数据
            System.out.println("## 心跳 1");
            //# 更新设备注册表 #
            deviceService.updateDeviceRegister(farmId, deviceCode,nowTimestamp);

            //# 更新在线统计 #
            deviceService.updateDeviceOnlineCount(farmId, deviceCode,todayDateText,nowTimestamp,todayDate);
        } else {
            System.out.println("## 心跳 2");
            //Step1:更新总状态表[device_online+1,device_offline-1,device_rate]，预先判断该数据是否存在，如果不存在要先添加记录，如果存在就修改数据
            //Step2:更新设备注册表[active_status=1,active_time=当前时间]
            //Step3:更新在线统计[]，预先判断该数据是否存在，如果不存在要先添加记录，如果存在就修改数据
            //Step4:将新设备编号加入缓存[]

            //# 更新设备注册表 #
            deviceService.updateDeviceRegister(farmId, deviceCode,nowTimestamp);

            //# 更新在线统计 #【重写完成】
            deviceService.updateDeviceOnlineCount(farmId, deviceCode, todayDateText, nowTimestamp, todayDate);

            //# 更新总状态表 # 【重写】
            deviceService.updateDeviceAllStatus(farmId, deviceCode,todayDateText,nowTimestamp,todayDate);

            //# 将新设备编号加入在线缓存 #
            cacheService.putDeviceToOnline(farmId, deviceCode,todayDateText);
        }

        //--设备在线统计表
        //--设备状态每日统计表
        //--设备和耳标总状态表

//        test1(farmId,deviceCode);

//        //# 心跳数加1，更改在线统计数据表（MySQL），更新Redis; #
//        deviceService.updateDeviceRegister(deviceCode);
//
//        //# 心跳数加1，更改在线统计数据表（MySQL），更新Redis; #
//        deviceService.updateDeviceOnline(deviceCode);
//
//        //# 处理状态数据 #
//        deviceService.updateFarmDeviceStatus(deviceCode);
    }

//    private void test1(String farmId, String deviceCode) {
////            String deviceCode = "202007239999869";
//////            Map map = new HashMap<>();
//////            map.put("name","栋舍");
//////            map.put("id","001");
//////            System.out.println("map>>"+map.toString());
////            String deviceRegHashKey = "device_register_"+deviceCode;
////            //为hash结构设置多个键值对(hmset)
//////        redisTemplate.opsForHash().putAll(deviceRegHashKey, map);
////
////
//////            Object actObj = redisTemplate.opsForHash().get(deviceCode, "act");
//////            System.out.println("actObj>>"+actObj);
////        Object nameObj = redisTemplate.opsForHash().get(deviceRegHashKey,"name");
////        System.out.println("nameObj>>"+nameObj.toString());
//
////        System.out.println("test1()");
////        redisTemplate.opsForSet().add("testSet", "java");
////
////        //定义一个set集合并设置集合值
////         SetOperations<String, String> set = redisTemplate.opsForSet();
////         set.add("set1","22");
////         set.add("set1","33");
////         set.add("set1","44");
////         // 通过key获取set集合中的对象值
////         Set<String> resultSet = redisTemplate.opsForSet().members("set1");
////         System.out.println("resultSet>>"+resultSet);
//        SetOperations<String, String> set = redisTemplate.opsForSet();
//        set.add(BizConst.SET_FARM_DEVICE_ONLINE_PREFIX + farmId, deviceCode);
//
//        Set<String> resultSet = redisTemplate.opsForSet().members(BizConst.SET_FARM_DEVICE_ONLINE_PREFIX + farmId);
//        System.out.println(BizConst.SET_FARM_DEVICE_ONLINE_PREFIX + farmId + " >>" + resultSet);
//    }

    /**
     * @Method : processDeviceTemp
     * @Description : 处理设备温度数据
     * @Params : [RawMap]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/2/15
     * @Time : 9:05
     */
//    @RabbitListener(queues = Const.QUEUE_DEVICE_TEMP)
    @RabbitHandler
    public void processDeviceTemp(Map DeviceTempMap) throws ParseException {
        System.out.println("333 <<<<<<<<<<<<<<< 处理设备温度数据>>" + DeviceTempMap.toString());
        JSONObject deviceJo = JSON.parseObject(DeviceTempMap.get("deviceTemp").toString());
        String deviceCode = deviceJo.getString("device");
        System.out.println("333 设备编号>>" + deviceCode);
        //--保存设备注册数据(MySQL,Redis)
        //--保存设备状态数据(MySQL,Redis)

        //--设备在线统计表
        //--设备状态每日统计表
        //--设备和耳标总状态表
        //获取牧场id
        String farmId = deviceService.getFarmIdByDeviceCode(deviceCode);

//        test1(farmId,deviceCode);
//        //# 处理状态数据 #
//        deviceService.updateFarmDeviceStatus(deviceCode);

        //# 判断在线缓存集合中是否存在该设备 #
        //# 如果存在：
        //      （1）更新设备在线数据[envtemp_total++,envtemp_first_time,envtemp_last_time]
        //      （2）更新注册设备数据[active_status=1,active_time]
        //# 如果不存在：
        //      （A） 更新总状态表；
        //      （B）更新设备注册表；
        //      （C）更新在线统计；
        //      （D）将设备好加入到在线缓存集合当中；
        String todayDateText = new DateUtil().getTodayDateText();
        Date todayDate = new Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        //判断该设备编号是否存在牧场缓存在线集合当中；
        if (cacheService.isExistDeviceOnlineSet(farmId, deviceCode,todayDateText)) {
//            System.out.println("## ++++");
            //# 更新在线统计 #
            deviceService.updateDeviceOnlineWithEnvtemp(farmId, deviceCode);
//            System.out.println("## ++++1");
            //# 更新设备注册表 #
            deviceService.updateDeviceRegister(farmId, deviceCode,nowTimestamp);
//            System.out.println("## ++++2");
        } else {
            //Step1:更新总状态表[device_online+1,device_offline-1,device_rate]，预先判断该数据是否存在，如果不存在要先添加记录，如果存在就修改数据
            //Step2:更新设备注册表[active_status=1,active_time=当前时间]
            //Step3:更新在线统计[]，预先判断该数据是否存在，如果不存在要先添加记录，如果存在就修改数据
            //Step4:将新设备编号加入缓存
//            System.out.println("## ----");
            //# 更新总状态表 #
            deviceService.updateEnvtempOfDeviceAllStatus(farmId, deviceCode);
//            System.out.println("## ----1");
            //# 更新设备注册表 #
            deviceService.updateDeviceRegister(farmId, deviceCode,nowTimestamp);
//            System.out.println("## ----2");
            //# 更新在线统计 #
            deviceService.updateDeviceOnlineWithEnvtemp(farmId, deviceCode);
//            System.out.println("## ----3");
            //# 将新设备编号加入在线缓存 #
            cacheService.putDeviceToOnline(farmId, deviceCode, todayDateText);
            System.out.println("## END");
        }
    }
}
