package com.huimv.eartag2.process2.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.repo.EartagEartagRegisterRepo;
import com.huimv.eartag2.common.mq.Const;
import com.huimv.eartag2.common.utils.BizConst;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.process2.producer.ProcessProducer;
import com.huimv.eartag2.process2.service.ICacheService;
import com.huimv.eartag2.process2.service.IDeviceService;
import com.huimv.eartag2.process2.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class EartagListener {
    @Autowired
    private ProcessProducer processProducer;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private IDeviceService deviceService;
    @Autowired
    private IEartagService eartagService;
    @Autowired
    private EartagEartagRegisterRepo eartagRegisterRepo;
    @Value("${redis.expire.eartag_online_set}")
    private Integer eartagOnlineSetExpire;

    /**
     * @Method : processRawdata
     * @Description : 订阅耳标数据队列数据
     * @Params : [RawMap]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/2/15
     * @Time : 8:55
     */
//    @RabbitListener(queues = Const.QUEUE_EARTAG)
    @RabbitHandler
    public void processRawdata(Map RawMap) throws ParseException {
        System.out.println("444 <<<<<<<<<<<<<<< 处理耳标数据>>" + RawMap.toString());
        JSONObject eartagJo = JSON.parseObject(RawMap.get("eartag").toString());
        //设备编码
        String deviceCode = eartagJo.getString("device");
//        System.out.println("444 设备编号>>" + deviceCode);
        //耳标号
        String earmark = eartagJo.getString("earmark");
//        System.out.println("444 耳标号>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>" + earmark);

        //--保存设备注册数据(MySQL,Redis)
        //--保存设备状态数据(MySQL,Redis)
        String todayDateText = new DateUtil().getTodayDateText();
        Date todayDate = new Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());

//        System.out.println("## eartagOnlineSetExpire>>"+eartagOnlineSetExpire);
        //--设备在线统计表
        //--设备状态每日统计表
        //--设备和耳标总状态表
        //获取牧场id
        String farmId = deviceService.getFarmIdByDeviceCode(deviceCode);
//        String askText = RawMap.get("askText").toString();
////        System.out.println("askText>>" + askText);
////        JSONObject dataJo = new JSONObject();
////        dataJo.put("name", "tony");
////        processProducer.sendHeartbeat(dataJo);
////        System.out.println("再发送心跳 >>>>>>>>>>>>>>>>>"+dataJo.toJSONString());
        //--保存耳标注册数据(MySQL,Redis)-processProducer
        //--保存耳标在线数据(MySQL,Redis)-processProducer
        //--保存耳标状态数据(MySQL,Redis)-processProducer

        //判断该设备编号是否存在牧场缓存在线集合当中；
        if (cacheService.isExistEartagOnlineSet(farmId, earmark,todayDateText)) {
            //Step1:更新耳标在线统计数据[上传记录数、最后上传时间、环境温度数据、耳标温度数据]
            //Step2:更新耳标注册表中统计数据[活动状态、活动状态更新时间]

            //Step1:更新耳标在线统计数据
            eartagService.updateEartagOnline(earmark,deviceCode,todayDateText,todayDate,nowTimestamp,eartagJo,farmId);

            //Step2:更新耳标注册表中统计数据
            eartagService.updateEartagRegister(earmark,deviceCode,nowTimestamp,farmId);
//            System.out.println("## 4.11111111111111111111111111");
        } else {
            //Step1:更新耳标注册数据
            //Step2:更新耳标在线数据
            //Step3:更新总状态数据
            //Step4:将耳标号加入到耳标在线集合当中
//            System.out.println("## 4.A");

            //Step1:更新耳标注册数据
            eartagService.updateEartagRegister(earmark,deviceCode,nowTimestamp,farmId);

            //Step2:更新耳标在线数据
            eartagService.updateEartagOnline(earmark,deviceCode,todayDateText,todayDate,nowTimestamp,eartagJo,farmId);

            //Step3:更新总状态数据
            eartagService.updateEartagOfAllStatus(earmark,deviceCode,todayDateText,farmId);

            //Step4:将耳标号加入到耳标在线集合当中
            cacheService.putEartagToOnlineInSet(farmId, earmark,todayDateText);
        }

    }
}
