package com.huimv.eartag2.process2.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.EartagEartagOnlineEntity;
import com.huimv.eartag2.common.dao.entity.EartagEartagRegisterEntity;
import com.huimv.eartag2.common.dao.entity.EartarFarmAllStatusEntity;
import com.huimv.eartag2.common.dao.repo.EartagEartagOnlineRepo;
import com.huimv.eartag2.common.dao.repo.EartagEartagRegisterRepo;
import com.huimv.eartag2.common.dao.repo.EartarFarmAllStatusRepo;
import com.huimv.eartag2.process2.service.ICacheService;
import com.huimv.eartag2.process2.service.IEartagService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Timestamp;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class EartagServiceImpl implements IEartagService {
    @Autowired
    private EartagEartagRegisterRepo eartagRegisterRepo;
    @Autowired
    private EartagEartagOnlineRepo eartagOnlineRepo;
    @Autowired
    private EartarFarmAllStatusRepo farmAllStatusRepo;
    @Autowired
    private ICacheService cacheService;

    @Override
    public void updateEartagRegister(String earmark, String deviceCode, Timestamp nowTimestamp, String farmId) {
        //# 判断耳标注册表是否存在；
        //      如果不存在就注册耳标；
        //      如果存在就更新活动状态（active_status）和活动状态更新时间（active_time）
        //
        EartagEartagRegisterEntity eartagRegisterEntity = eartagRegisterRepo.getOneByEarmark(earmark,farmId);
        if(eartagRegisterEntity == null){
            //#创建耳标注册记录#
            EartagEartagRegisterEntity newEartagRegisterEntity = new EartagEartagRegisterEntity();
            newEartagRegisterEntity.setEarmark(earmark);
            newEartagRegisterEntity.setFirstTime(nowTimestamp);
            newEartagRegisterEntity.setLastTime(nowTimestamp);
            newEartagRegisterEntity.setFirstDevice(deviceCode);
            newEartagRegisterEntity.setBelongDevice(deviceCode);
            newEartagRegisterEntity.setLastDevice(deviceCode);
            newEartagRegisterEntity.setRegisterTime(nowTimestamp);
            newEartagRegisterEntity.setRegisterType(1);
            newEartagRegisterEntity.setFarmId(farmId);
            newEartagRegisterEntity.setActiveStatus(1);
            newEartagRegisterEntity.setActiveTime(nowTimestamp);
            newEartagRegisterEntity.setLiveStatus(1);
            eartagRegisterRepo.saveAndFlush(newEartagRegisterEntity);
            //更新耳标注册消息缓存
            cacheService.putEartagRegister(earmark,newEartagRegisterEntity);
        }else{
            eartagRegisterEntity.setActiveStatus(1);
            eartagRegisterEntity.setActiveTime(nowTimestamp);
            eartagRegisterRepo.saveAndFlush(eartagRegisterEntity);
            //更新耳标注册消息缓存
            cacheService.putEartagRegister(earmark,eartagRegisterEntity);
        }
    }

    @Override
    public void updateEartagOnline(String earmark, String deviceCode, String todayDateText, Date todayDate, Timestamp nowTimestamp, JSONObject eartagJo, String farmId) {
        //# 判断耳标在线统计数据是否存在
        //      存在，则更新上传记录数、最后上传时间、环境温度，耳根温度，并将上面数据更新到缓存数据中；
        //      不存在，则创建耳标在线记录，并更新到缓存数据中；
        //耳根温度
        String earTemp =  eartagJo.getString("earTemp1");
        //环境温度
        String envTemp =  eartagJo.getString("envTemp");
        EartagEartagOnlineEntity eartagOnlineEntity = eartagOnlineRepo.getOneByEarmarkAndFarmId(earmark,todayDateText,farmId);
        if(eartagOnlineEntity == null){
            //# 不存在
            EartagEartagOnlineEntity newEartagOnlineEntity = new EartagEartagOnlineEntity();
            newEartagOnlineEntity.setEarmark(earmark);
            newEartagOnlineEntity.setTotal(1);
            newEartagOnlineEntity.setFirstTime(nowTimestamp);
            newEartagOnlineEntity.setLastTime(nowTimestamp);
            newEartagOnlineEntity.setEnvTemp(envTemp);
            newEartagOnlineEntity.setEartagTemp(earTemp);
            newEartagOnlineEntity.setAddDate(todayDate);
            newEartagOnlineEntity.setFarmId(farmId);
            eartagOnlineRepo.saveAndFlush(newEartagOnlineEntity);
            //更新耳标在线统计
            cacheService.putEartagOnlineCount(earmark,newEartagOnlineEntity);
        }else{
            //# 存在
            int total = eartagOnlineEntity.getTotal();
            eartagOnlineEntity.setTotal(++total);
            eartagOnlineEntity.setLastTime(nowTimestamp);
            eartagOnlineEntity.setEnvTemp(envTemp);
            eartagOnlineEntity.setEartagTemp(earTemp);
            eartagOnlineRepo.saveAndFlush(eartagOnlineEntity);
            //更新耳标在线统计
            cacheService.putEartagOnlineCount(earmark,eartagOnlineEntity);
        }
    }

    @Override
    public void updateEartagOfAllStatus(String earmark, String deviceCode,String todayDateText, String farmId) {
        //# 判断设备和耳标总状态表是否存在
        //      存在：更新在线耳标数量；
        //      不存在：创建状态
        EartarFarmAllStatusEntity farmAllStatusEntity = farmAllStatusRepo.getOneByFarmIdAndCreateDate(farmId,todayDateText);
        if(farmAllStatusEntity == null){
            //#获取耳标注册数量#
            int eartagTotal =  getEartagRegTotalByFarmId(farmId);
            //#总状态记录不存在；
            EartarFarmAllStatusEntity newFarmAllStatusEntity = new EartarFarmAllStatusEntity();
            newFarmAllStatusEntity.setEartagTotal(eartagTotal);
            newFarmAllStatusEntity.setEartagOnline(1);
            farmAllStatusRepo.saveAndFlush(newFarmAllStatusEntity);
        }else{
            //#存在；
            int total = farmAllStatusEntity.getEartagOnline();
            farmAllStatusEntity.setEartagOnline(++total);
            farmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
        }
    }

    //读取耳标注册数据
    @Override
    public Integer getEartagRegTotalByFarmId(String farmId){
        int eartagTotal = 0;
        List<Object[]> eartagRegList = eartagRegisterRepo.getEartagCountByFarmId(farmId);
        Object[] eartagRegObj = (Object[]) eartagRegList.get(0);
        //处于工作状态的耳标总数
        return Integer.parseInt(eartagRegObj[0].toString());
    }

    @Override
    public int getEartagOnelineTotal(String farmId, String todayDateText) {
        cacheService.getEartagOnlineTotalFromCache(farmId,todayDateText);
        return 0;
    }
}
